from calle import Calle
from quitanieves import Quitanieves
from copo import Copo
import random


class Ciudad:
    def __init__(self, height: int, width: int):
        self.alto = height
        self.ancho = width
        self.calles = None
        self.quitanieves = None
        
    @property
    def alto(self):
        return self.__alto
    
    @alto.setter
    def alto(self, height: int):
        if height >= 5:
            self.__alto = height
        else:
            self.__alto = 5

    @property
    def ancho(self):
        return self.__ancho

    @ancho.setter
    def ancho(self, width: int):
        if width >= 5:
            self.__ancho = width
        else:
            self.__ancho = 5

    def crear_calles(self):
        """ Crea la tupla de calles de la ciudad"""
        # Convertiremos la lista en tupla al final
        aux = []
        # Horizontal
        for s in range(self.alto):
            temp = random.randint(-5, 4)
            length = random.randint(5, self.ancho)
            aux.append(Calle(True, 0, s, length, temp))
        # Vertical
        for s in range(self.ancho):
            temp = random.randint(-5, 4)
            length = random.randint(5, self.alto)
            aux.append(Calle(False, s, 0, length, temp))

        self.calles = tuple(aux)

    def crear_quitanieves(self):
        """ Crea los quitanieves de la ciudad"""
        # Convertiremos la lista en tupla al final
        aux = []
        for street in self.calles:
            x = street.x_inicial
            y = street.y_inicial
            fuel = random.randint(1, 20)
            aux.append(Quitanieves(x, y, fuel))
        self.quitanieves = tuple(aux)

    def nevar(self, number: int):
        """ Hace que nieve"""
        for counter in range(number):
            # Seleccionamos la calle
            street = self.calles[random.randrange(len(self.calles))]
            # Seleccinamos una posición aleatoria en la calle
            pos = random.randrange(street.longitud)
            if street.horizontal:
                x = pos
                y = street.y_inicial
            else:
                x = street.x_inicial
                y = pos
            # Creamos el copo
            sn = Copo(x, y, street.temperatura)
            street.copos.append(sn)

    def calles_a_limpiar(self)-> str:
        """ Devuelve qué calles hay que limpiar"""
        res = ""
        for street in self.calles:
            if street.necesita_limpieza():
                res = res + str(street) + "\n"
        return res

    def pasar_quitanieves(self):
        """ limpia las calles """
        for index in range(len(self.calles)):
            # Variable auxiliar para que el código se entienda mejor
            street = self.calles[index]
            if street.necesita_limpieza():
                # El primer quitanieves está en la primera calle y así
                # sucesivamente
                snowplow = self.quitanieves[index]
                finish = False
                while not finish:
                    snowplow.gasolina -= 1
                    # Para evitar problemas al quitar elementos de la lista,
                    # empezamos siempre por el último
                    for ind in range(len(street.copos) - 1, -1, -1):
                        if (street.copos[ind].x == snowplow.x and
                                street.copos[ind].y == snowplow.y):
                            del(street.copos[ind])
                    if street.horizontal:
                        # Si la calle es horizontal, nos movemos en x
                        snowplow.x = snowplow.x + 1
                        # Si hemos llegado al final de la calle, terminamos
                        finish = snowplow.x > street.longitud
                    else:
                        # En las horizontales nos movemos en y
                        snowplow.y = snowplow.y + 1
                        finish = snowplow.y > street.longitud
                    # Si no tenemos gasolina o estamos al final de la calle,
                    # terminamos
                    finish = finish or snowplow.gasolina <= 0

